@@ -3,13 +3,14 @@
 /* from Peter C. Gutmann's implementation as found in */
 /* Applied Cryptography by Bruce Schneier */
 /* This code is in the public domain */
-/* $Id: sha1.h 7777 2006-06-01 17:37:15Z matt $ */
+/* $Id$ */
 
 #ifndef SHA_H
 #define SHA_H
 
 #include <stdlib.h>
 #include <stdio.h>
+#include "discid/discid.h" /* for LIBDISCID_INTERNAL */
 
 /* Useful defines & typedefs */
 typedef unsigned char SHA_BYTE;	/* 8-bit quantity */
@@ -22,16 +23,16 @@ typedef struct {
     SHA_LONG digest[5];		/* message digest */
     SHA_LONG count_lo, count_hi;	/* 64-bit bit count */
     SHA_BYTE data[SHA_BLOCKSIZE];	/* SHA data buffer */
-    int local;			/* unprocessed amount in data */
+    size_t local;			/* unprocessed amount in data */
 } SHA_INFO;
 
-void sha_init(SHA_INFO *);
-void sha_update(SHA_INFO *, SHA_BYTE *, int);
-void sha_final(unsigned char [20], SHA_INFO *);
+LIBDISCID_INTERNAL void sha_init(SHA_INFO *);
+LIBDISCID_INTERNAL void sha_update(SHA_INFO *, SHA_BYTE *, size_t);
+LIBDISCID_INTERNAL void sha_final(unsigned char [20], SHA_INFO *);
 
-void sha_stream(unsigned char [20], SHA_INFO *, FILE *);
-void sha_print(unsigned char [20]);
-char *sha_version(void);
+LIBDISCID_INTERNAL void sha_stream(unsigned char [20], SHA_INFO *, FILE *);
+LIBDISCID_INTERNAL void sha_print(unsigned char [20]);
+LIBDISCID_INTERNAL char *sha_version(void);
 
 #define SHA_VERSION 1
 
